unit BOM;

interface

uses
  classes
  , contnrs
  ;

type
  TUserAction = class (TObject)
  private
    FName: string;
    FOID: Integer;
  public
    procedure Execute;
    property Name: string read FName write FName;
    property OID: Integer read FOID write FOID;
  end;

  TUserType = class (TObject)
  private
    FName: string;
    FOID: Integer;

    function GetActions: TList;

  public

    procedure AddAction(Action: TUserAction);
    procedure RemAction(Action: TUserAction);

    property Actions: TList read GetActions;
    property Name: string read FName write FName;
    property OID: Integer read FOID write FOID;
  end;

  TUser = class (TObject)
  private

    FFullName: string;
    FOID: Integer;
    FPassword: string;
    FUserName: string;

    function GetUserType: TUserType;
    procedure SetUserType(Value: TUserType);

  public

    property FullName: string read FFullName write FFullName;
    property OID: Integer read FOID write FOID;
    property Password: string read FPassword write FPassword;
    property UserName: string read FUserName write FUserName;
    property UserType: TUserType read GetUserType write SetUserType;

  end;

  // Would be a good idea to override the Add method to make sure I'm adding
  // objects of the correct type

  TUserActionList = class (TObjectList)
  end;

  TUserTypeList = class (TObjectList)
  end;

  TUserList = class (TObjectList)
  end;

// constants for relationship IDs (RelID values)
const
  riUserType = 100;
  riTypeAction = 101;

// Define a few global variables (not recommended for production systems!)
var
  UserList: TUserList;
  UserTypeList: TUserTypeList;
  UserActionList: TUserActionList;

implementation

uses
  sysutils
  , uRelMan
  ;

//******************* PopulateObjects *************************
// Generate some test data
procedure PopulateObjects;
var
  oid, n, m, nActions: integer;
  usrTemp: TUser;
  usrTypeTemp: TUserType;
  usrActionTemp: TUserAction;
begin
  oid := 0;
  // Create 10 user actions
  UserActionList := TUserActionList.Create;
  for n := 0 to 9 do
  begin
    usrActionTemp := TUserAction.Create;
    usrActionTemp.Name := 'UserAction' + IntToStr(n);
    usrActionTemp.OID := oid;
    Inc(oid);
    UserActionList.Add(usrActionTemp);
  end;
  // Create 10 user types
  UserTypeList := TUserTypeList.Create;
  for n := 0 to 9 do
  begin
    usrTypeTemp := TUserType.Create;
    usrTypeTemp.Name := 'UserType' + IntToStr(n);
    usrTypeTemp.OID := oid;
    Inc(oid);
    // Add some actions
    nActions := Random(UserActionList.Count);
    for m := 0 to nActions do
    begin
      usrActionTemp := TUserAction(UserActionList.Items[Random(UserActionList.Count)]);
      while (usrTypeTemp.Actions.IndexOf(usrActionTemp) <> -1) do
      begin
        usrActionTemp := TUserAction(UserActionList.Items[Random(UserActionList.Count)]);
      end;
      usrTypeTemp.AddAction(usrActionTemp);
    end;
    UserTypeList.Add(usrTypeTemp);
  end;
  // Create 10 users
  UserList := TUserList.Create;
  for n := 0 to 9 do
  begin
    usrTemp := TUser.Create;
    usrTemp.FullName := 'A. User' + IntToStr(n);
    usrTemp.OID := oid;
    Inc(oid);
    usrTemp.Password := 'password' + IntToStr(n);
    usrTemp.UserName := 'User' + IntToStr(n);
    usrTemp.UserType := TUserType(UserTypeList.Items[Random(UserTypeList.Count)]);
    UserList.Add(usrTemp);
  end;
end;

{
********************************* TUserAction **********************************
}
procedure TUserAction.Execute;
begin
end;

{ TUserType }

procedure TUserType.AddAction(Action: TUserAction);
begin
  gRelMan.AddRelationship(Self, Action, riTypeAction);
end;

function TUserType.GetActions: TList;
begin
  Result := gRelMan.FindObjects(Self, nil, riTypeAction);
end;

procedure TUserType.RemAction(Action: TUserAction);
begin
  gRelMan.RemRelationship(Self, Action, riTypeAction);
end;

{ TUser }

function TUser.GetUserType: TUserType;
var
{  lstTemp: TList;}
  objTemp: TObject;
begin
  Result := nil;
{  lstTemp := nil;
  try} { ... finally }
{    lstTemp := gRelMan.FindObjects(Self, nil, riUserType);
    if ((lstTemp.Count > 0) and (TObject(lstTemp.Items[0]) is TUserType)) then
      Result := TUserType(lstTemp.Items[0]);
  finally
    lstTemp.Free;
  end;}  { try ... finally }
  objTemp := gRelMan.FindObject(Self, nil, riUserType);
  if (Assigned(objTemp) and (objTemp is TUserType)) then
    Result := TUserType(objTemp);
end;

procedure TUser.SetUserType(Value: TUserType);
var
  utTemp: TUserType;
begin
  // Get the current UserType from the relationship manager. Cache it in a
  // local variable.
  utTemp := Self.UserType;
  if (Value <> utTemp) then
  begin
    // Remove existing relationship
    if Assigned(utTemp) then
    begin
      gRelMan.RemRelationship(Self, utTemp, riUserType);
    end;
    // Add the new relationship
    gRelMan.AddRelationship(Self, Value, riUserType);
  end;
end;

initialization
begin
  PopulateObjects;
end;

end.
